package com.pj.biz;

import java.util.List;

import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFiling;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFilingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: re_mini_user_car_filing -- 微信端用户与车辆备案中间表
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/ReMiniUserCarFiling/")
public class ReMiniUserCarFilingController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    ReMiniUserCarFilingService reMiniUserCarFilingService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(ReMiniUserCarFiling.PERMISSION_CODE)
    @Transactional(rollbackFor = Exception.class)
    public AjaxJson add(ReMiniUserCarFiling r) {
        reMiniUserCarFilingService.add(r);
        return AjaxJson.getSuccessData(r);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(ReMiniUserCarFiling.PERMISSION_CODE)
    public AjaxJson delete(Long id) {
        reMiniUserCarFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(ReMiniUserCarFiling.PERMISSION_CODE)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(ReMiniUserCarFiling.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(ReMiniUserCarFiling.PERMISSION_CODE)
    public AjaxJson update(ReMiniUserCarFiling r) {
        reMiniUserCarFilingService.update(r);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        ReMiniUserCarFiling r = reMiniUserCarFilingService.getById(id);
        return AjaxJson.getSuccessData(r);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        List<ReMiniUserCarFiling> list = reMiniUserCarFilingService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


}
